/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BUCKET_H
#define BUCKET_H

#include "utils.h"
#include "cost.h"
#include "type.h"
#include "costtype.h"
#include "point.h"

namespace SpritePack {

	struct ImageInfo;

	struct Bucket
	{
		struct Item
		{
			Id id;
			int x;
			int y;

			Item(Id = -1, int x = -1, int y = -1);
			Item(Id, const Point&);
			operator Id() const;
		};
		typedef vector<Item> Items;

		typedef int Id;

	public:
		Bucket();

		Bucket::Id id() const;

		const Items& items() const;

		Type type() const;
		
		Quality quality() const;

		int interlaceType() const;

		void setInterlaceType(int);

		void setType(Type);

		void setQuality(Quality);

		void setItems(const Items&);

		void setItems(const Bucket&);

		void clear();

		void add(Id, int x, int y);

		void add(Id, const Point&);

		void addItems(const Items&);

		void addItems(const Bucket&);

		void assignItems(const Items&);

		void setInfo(const Bucket&);

		const string& fileName() const;

		void setId(const Bucket::Id&) const;

	private:
		Items m_items;

		Type m_type;

		Quality m_quality;

		int m_interlaceType;

		void modify();

		bool isUpToDate(CostType) const;

	private:
		friend class Base;

		mutable Bucket::Id m_id;

		mutable FileSize m_fileSize;

		mutable Cost m_geometryCost;

		mutable Cost m_compressionCost;

		mutable bool m_isUpToDateGeometry;

		mutable bool m_isUpToDateCompression;	

		mutable string m_fileName;  

		mutable string m_geometryMoveName;

		mutable string m_compressionMoveName;

	private:
		void setUpToDate(CostType, bool = true) const;

		static Bucket::Id generateId();

	};

	typedef list<Bucket> Buckets;

	Buckets& operator+=(Buckets&, const Bucket&);

}

#endif /* BUCKET_H */