/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef BUCKET_MOVE_H
#define BUCKET_MOVE_H

#include "utils.h"
#include "bucket.h"
#include "costtype.h"
#include "direction.h"
#include "move.h"
#include "geometry.h"

namespace SpritePack {

	class BucketMove : public MoveImpl
	{
	public:
		const Bucket * input;
		Bucket * output;

		Bucket result;

		CostType costType;

		Cost calcResultCost();

	public:
		virtual void beforeCalc();

	public:
		virtual void setInput(Bucket *);

		virtual void apply();

		virtual bool isImprovement();

		virtual void clear();

		BucketMove(CostType);
				
		virtual ~BucketMove();

		vector<Id> getImageIds() const;
	};

	class GeometryBucketMove : public BucketMove
	{
	public:
		GeometryBucketMove();
		bool checkForCollisions(bool throwError = true);
	};

	class DirectionGeometryBucketMove : public GeometryBucketMove
	{
	public:
		struct DecreasingNormalWidthCmp
		{
			DirectionGeometryBucketMove * move;

			DecreasingNormalWidthCmp(DirectionGeometryBucketMove *);

			bool operator()(Id, Id) const;
		};

		struct DecreasingNormalHeightCmp
		{
			DirectionGeometryBucketMove * move;

			DecreasingNormalHeightCmp(DirectionGeometryBucketMove *);

			bool operator()(Id, Id) const;
		};

		struct IncreasingNormalWidthCmp
		{
			DirectionGeometryBucketMove * move;

			IncreasingNormalWidthCmp(DirectionGeometryBucketMove *);

			bool operator()(Id, Id) const;
		};

		struct IncreasingNormalHeightCmp
		{
			DirectionGeometryBucketMove * move;

			IncreasingNormalHeightCmp(DirectionGeometryBucketMove *);

			bool operator()(Id, Id) const;
		};

	public:
		DirectionGeometryBucketMove();

		Direction direction;

		Geometry normalGeometry(Id) const;

		Geometry normalGoemetry(const Geometry&) const;

		Point normalPoint(const Point&) const;

		Point normalPoint(int x, int y) const;

		int normalWidth(Id) const;

		int normalHeight(Id) const;

		int maxNormalWidth() const;

		int maxNormalHeight() const;

		int minNormalWidth() const;

		int minNormalHeight() const;

		Bucket::Items normalItems(const Bucket::Items&) const;

	public:
		virtual void setInput(Bucket * bucket, Direction direction);

	protected:
		mutable int m_maxNormalWidth;

		mutable int m_maxNormalHeight;

		mutable int m_minNormalWidth;

		mutable int m_minNormalHeight;
	};

	class CompressionBucketMove : public BucketMove
	{
	public:
		CompressionBucketMove();
	};

} /* namespace SpritePack */

#endif /* BUCKET_MOVE_H */
