/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DH_H
#define DH_H

#include "bucketmove.h"

namespace SpritePack {

	class DH : public DirectionGeometryBucketMove
	{
	public:
		struct Level
		{
			vector<Id> items;

			int freeSpace;

			int height;

			Level(int freeSpace = 0);
		};

		typedef vector<Level> Levels;
		typedef vector<Level>::iterator LevelsIterator;

	public:
		bool twoFit;

	public:
		DH();
		virtual void setInput(Bucket *, Direction, bool twoFit = false);

	public:
		virtual LevelsIterator chooseLevel(Levels*, const LevelsIterator& lastItr, Id) const = 0;

	public:
		virtual Cost calc();

		void normalCalc(Levels *);
		void twoFitCalc(Levels *);
		Cost generateResult(const Levels&);
	};

	class FFDH : public DH
	{
	public:
		virtual string name() const;

	protected:
		virtual LevelsIterator chooseLevel(Levels*, const LevelsIterator& lastItr, Id) const;
	};

	class NFDH : public DH 
	{
	public:
		virtual string name() const;

	protected:
		virtual LevelsIterator chooseLevel(Levels*, const LevelsIterator& lastItr, Id) const;
	};

	class BFDH : public DH
	{
	public:
		virtual string name() const;

	protected:
		virtual LevelsIterator chooseLevel(Levels*, const LevelsIterator& lastItr, Id) const;
	};

} /* namespace SpritePack */


#endif /* DH_H */