/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "jpegcompressor.h"
#include "base.h"

namespace SpritePack {

	JPEGCompressor::JPEGCompressor()
	: m_interlaceType(0)
	{}

	Cost JPEGCompressor::calc()
	{
		if (!base || !input)
			return Cost::infinity();

		if (!base->hasTransparency(*input) && (base->allowLossyTypeChange || base->hasOnlyLossless(*input)))
		{
			result.setItems(*input);
			result.setType(Jpeg);
			result.setInterlaceType(m_interlaceType);
			result.setQuality(base->jpegQuality);

			return calcResultCost();
		}

		return Cost::infinity();
	}

	string JPEGCompressor::name() const
	{
		if (base)
		{
			return "JPEG(" + toString(base->jpegQuality) + ", " + toString(m_interlaceType) + ")";
		}
		else
		{
			return "JPEG(?, ?)";
		}
	}

	void JPEGCompressor::setInput(Bucket * bucket, int interlaceType)
	{
		CompressionBucketMove::setInput(bucket);
		m_interlaceType = interlaceType;
	}

} /* namespace SpritePack */