/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "log.h"

namespace SpritePack {

	fstream * Log::m_fstream = 0;

	void Log::openLogFile(const string& fileName)
	{
		closeLogFile();
		m_fstream = new fstream(fileName.c_str(), fstream::out | fstream::trunc | fstream::binary);
		if (!m_fstream->is_open()) 
			closeLogFile();
	}

	void Log::closeLogFile()
	{
		if (m_fstream) {
			m_fstream->close();
			delete m_fstream;
			m_fstream = 0;
		}
	}

	Log::Log(ostream& o, bool printTime)
		: m_stream(&o)
	{
		if (printTime)
		{
			operator<<(Time::toString());
		}
	}

	Log::Log()
		: m_stream(0)
	{
		m_stream = &cout;

		operator<<(Time::toString());
	}

	Log::~Log()
	{
		if (m_stream)
		{
			(*m_stream) << endl;
			(*m_stream).flush();
		}

		if (m_fstream)
		{
			const string endl = "\r\n";
			(*m_fstream) << endl;
			(*m_fstream).flush();
		}
	}


} /* namespace SpritePack */
