/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "spritepack.h"

namespace SpritePack
{
	set<string> parseAlgorithms(const string& line)
	{
		set<string> result;
		vector<string> elems;
		split(toLower(line), ',', elems);
		for (string item : elems)
			result.insert(trim(item));
		return result;
	}

	vector<double> parseSpeedMbpsTable(const string& line)
	{
		vector<double> result;
		result.push_back(0);
		vector<string> elems;
		split(toLower(line), ',', elems);
		for (string item : elems)
			result.push_back(fromString<double>(trim(item)));
		return result;
	}

	void init(SpritePack* spritePack, int argc, char* argv[])
	{
		if (!spritePack)
			return;

		vector<double> defaultSpeedMbpsTable;
		defaultSpeedMbpsTable.push_back(0);
		defaultSpeedMbpsTable.push_back(3.5);
		defaultSpeedMbpsTable.push_back(2.5);
		defaultSpeedMbpsTable.push_back(1.5);

		spritePack->inputDir = argc > 2 ? argv[1] : "input";
		spritePack->outputDir = argc > 2 ? argv[2] :  "output";
		spritePack->startupMs = argc > 3 ? fromString<int>(argv[3]) : 200;
		spritePack->speedMbpsTable = argc > 4 ? parseSpeedMbpsTable(argv[4]) : defaultSpeedMbpsTable;
		spritePack->k = argc > 5 ? trim(argv[5]) : "10%";
		spritePack->allowLossyTypeChange = argc > 6 ? fromString<int>(argv[6]) : false;
		spritePack->jpegQuality = argc > 7 ? fromString<int>(argv[7]) : 90;
		spritePack->pngCompressionLevel = argc > 8 ? fromString<int>(argv[8]) : 9;

		const set<string> algorithms = parseAlgorithms(argc > 9 ? argv[9] : "mbl,bl,vhtl,twofit,ffdh,bfdh");

		spritePack->useMBL = algorithms.find("mbl") != algorithms.end();
		spritePack->useBL = algorithms.find("bl") != algorithms.end();
		spritePack->useVHTL = algorithms.find("vhtl") != algorithms.end();
		spritePack->useTwoFit = algorithms.find("twofit") != algorithms.end();
		spritePack->useFFDH = algorithms.find("ffdh") != algorithms.end();
		spritePack->useNFDH = algorithms.find("nfdh") != algorithms.end();
		spritePack->useBFDH = algorithms.find("bfdh") != algorithms.end();

		spritePack->pngFilter = argc > 10 ? fromString<int>(argv[10]) : -1;
		spritePack->postprocessCmd = argc > 11 ? argv[11] : "";

	#ifdef __DEBUG__
		spritePack->startupMs = 200;
		spritePack->speedMbpsTable = defaultSpeedMbpsTable;
		spritePack->k = "1%";
		spritePack->pngFilter = 5;
		spritePack->useMBL = false;
		spritePack->useBL = true;
		spritePack->useTwoFit = false;
		spritePack->useVHTL = false;
		spritePack->useFFDH = false;
		spritePack->useNFDH = false;
		spritePack->useBFDH = false;
		spritePack->jpegQuality = 90;
		spritePack->allowLossyTypeChange = false;
		spritePack->inputDir = "input";
		spritePack->outputDir = "output";
		spritePack->postprocessCmd = "";
	#endif

		spritePack->checkInputParams();
		spritePack->checkInputDir();
		spritePack->checkOutputDir();

	}
} // namespace SpritePack

int main(int argc, char* argv[])
{
	Magick::InitializeMagick(argv[0]);

	SpritePack::SpritePack spritePack;



	SpritePack::init(&spritePack, argc, argv);
	//spritePack.openLogFile();
	spritePack.load();
	spritePack.make();

#ifdef __DEBUG__
	if (argc == 1)
	{
		system("pause");
	}
#endif

	return 0;
}