/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MERGE_MOVE_H
#define MERGE_MOVE_H

#include "utils.h"
#include "move.h"
#include "bucket.h"
#include "costtype.h"

namespace SpritePack {

	struct MergeMove : public MoveImpl
	{
	public:
		bool compress;

		Buckets * buckets;

		Buckets::iterator itr1;

		Buckets::iterator itr2;

		Bucket result;

	public:
		static Buckets::iterator inputItr1(const Move&);

		static Buckets::iterator inputItr2(const Move&);

		static Buckets::iterator resultItr(const Move&);

	public:
		MergeMove();

		virtual ~MergeMove();

	public:
		virtual Cost calc();
	
		virtual void apply();

		virtual string name() const;

		virtual bool isImprovement();

	public:
		void setInput(bool compress, Buckets* buckets, const Buckets::iterator&, const Buckets::iterator&);

	private:
		CostType costType() const;

		Buckets::iterator m_resultItr;
	};

	class MergeMoves
	{
	public:
		MergeMoves();

		void removeDependencies(const Move&);

		void addDependencies(const Move&);

		MergeMoves& operator+=(const Move&);
		
		Move bestMove() const;

		void init(Base *, Buckets *, bool compress);

		bool apply(const Move&, bool improvementOnly = false);

		bool applyIfImprovement(const Move&);

	private:
		struct MoveCmp
		{
			bool operator()(const Move& a, const Move& b) const;
		};

		typedef multimap<Bucket::Id, Move> Map;

		typedef set<Move, MoveCmp> Set;

		Set m_moves;

		Map m_map;

		Base * m_base;

		Buckets * m_input;

		bool m_compress;
	};


} /* namespace SpritePack */

#endif /* MERGE_MOVE_H */

