/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef MOVE_H
#define MOVE_H

#include "utils.h"
#include "cost.h"

#include "refptr.h"

namespace SpritePack {

	class Base;

	class MoveImpl : public RefCounted
	{
	public:
		Base * base;

		MoveImpl(Base * = 0);

		virtual ~MoveImpl();

	public:
		virtual Cost cost();

	public:
		virtual void beforeCalc() {}

		virtual Cost calc() = 0;

		virtual void afterCalc() {}
		
		virtual void apply() = 0;

		virtual string name() const = 0;

		virtual bool isImprovement() = 0;

	protected:
		virtual void setBase(Base *);

		bool m_isCalculated;

		Cost m_cost;

		friend class Base;
	};

	class NullMove : public MoveImpl
	{
	public:
		NullMove(Cost);

		virtual Cost calc();

		virtual void apply();

		virtual string name() const;

		virtual bool isImprovement();
	};

	class Move
	{
	public:
		bool apply(bool improvementOnly = false) const;

		bool applyIfImprovement() const;

		Cost cost() const;

		string name() const;

		bool operator<(const Move&) const;

		Move(const RefPtr<MoveImpl>& = 0);

		Move(Cost cost);

		bool isNull() const;

		bool isGood(bool improvementOnly = false) const;

		bool isImprovement() const;

		MoveImpl * impl() const;

	private:
		RefPtr<MoveImpl> m_impl;
	};

	typedef list<Move> Moves;

	Move bestMove(const Moves& moves);

	Moves& operator+=(Moves&, const Move&);

} /* namespace MOVE_H */

#endif /* MOVE_H */

