/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "number.h"
#include "utils.h"

namespace SpritePack {

	Number::Number(Type _value)
		: value(_value) 
	{}

	Number::Number(const Number& l)
		: value(l.value) 
	{}		

	Number::operator Number::Type() const
	{ 
		return value; 
	}

	Number::Type Number::infinityValue()
	{
		static Type v = 0;
		if (!v)
		{
			bool hasInfinity = numeric_limits<Type>::has_infinity; 
			if (hasInfinity)
			{
				v = numeric_limits<Type>::infinity();
			}
			else
			{
				v = numeric_limits<Type>::max();
			}
		}
		return v;
	}

	Number& Number::operator+=(const Number& a)
	{
		if (!isInfinity() && !a.isInfinity())
		{
			value += a.value;
		}
		if (a.isInfinity())
		{
			value = a.value;
		}
		return *this;
	}

	Number& Number::operator-=(const Number& a)
	{
		if (!isInfinity() && !a.isInfinity())
		{
			value -= a.value;
		}
		if (a.isInfinity())
		{
			value = a.value;
		}
		return *this;
	}

	Number Number::operator+(const Number& a) const
	{
		Number l(*this);
		l += a;
		return l;
	}

	Number Number::operator-(const Number& a) const
	{
		Number l(*this);
		l -= a;
		return l;
	}

	bool Number::operator<(const Number& a) const
	{
		return value < a.value;
	}

	string Number::toString() const
	{
		if (!isInfinity())
		{
			return SpritePack::toString(value);
		}
		else
		{
			return "infinity";
		}
	}

	bool Number::isInfinity() const
	{
		return value == infinityValue();
	}

	Number Number::infinity()
	{
		return infinityValue();
	}

} /* namespace SpritePack */
