/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
#include "pngcompressor.h"

namespace SpritePack {

	PNGCompressor::PNGCompressor()
		: m_compressionLevel(5), m_filter(1)
	{}

	Cost PNGCompressor::calc()
	{
		if (!input)
			return Cost::infinity();

		const Quality quality = (m_compressionLevel % 10) * 10 + m_filter % 10;

		result.setItems(*input);
		result.setType(Png);
		result.setQuality(quality);

		return calcResultCost();
	}

	string PNGCompressor::name() const
	{
		return "PNG(" + toString(m_compressionLevel) + ", " + toString(m_filter) + ")";
	}

	void PNGCompressor::setInput(Bucket * bucket, int compressionLevel, int filter)
	{
		CompressionBucketMove::setInput(bucket);
		m_filter = filter;
		m_compressionLevel = compressionLevel;
	}

} /* namespace SpritePack */