/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SPRITE_PACK_H
#define SPRITE_PACK_H

#include "base.h"

namespace SpritePack {

	class MergeMoves;

	class SpritePack : public Base
	{
	public:
		virtual void make();

		virtual Moves compressionMoves(Bucket*);

		virtual Moves directionGeometryMoves(Bucket * bucket, Direction);

		virtual Moves directionLessGeometryMoves(Bucket * bucket);

	public:
		virtual void makeFull();

		virtual void makeOne();

		void showProgress(int pos, int max);

		template <class Function> int initClasses(list< pair<Buckets, MergeMoves> > *, Function);

		void dumpInfo(const Buckets&, const BeforePostProcessInfo& = BeforePostProcessInfo());

	public:
		SpritePack();

		int lastProgressPos;
		int lastProgressMax;

		double preprocessingFinishedTime;
		double bucketAssignmentFinishedTime;
		double bucketPrecompressionFinishedTime;
		double bucketMergingFinishedTime;
		double postprocessingFinishedTime;
	};

	/* implementation */

	template <class Function> int SpritePack::initClasses(list< pair<Buckets, MergeMoves> > * classes, Function fun)
	{
		if (!classes)
			return 0;

		Buckets buckets;
		for (const ImageInfo& info: imageInfos)
		{
			if (fun(info))
			{
				Bucket bucket;
				bucket.add(info, 0, 0);
				bucket.id(); //ensure bucket::id
				buckets += bucket;
			}
		}
		if (!buckets.empty())
		{
			classes->push_back(make_pair(buckets, MergeMoves()));
		}
		return (int)buckets.size();
	}

} /* namespace SpritePack */

#endif /* SPRITE_PACK_H */
