/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef UTILS_H
#define UTILS_H

#include "thirdparty.h"

namespace SpritePack {

	typedef int Quality;

	typedef int Id;

	typedef long long int FileSize;

	typedef long long int Area;

	class Base;
	
	struct Bucket;

	struct Type;

	class Move;

	list<string> readdir(const string& directory, bool recursive = true);

	string pathJoin(const string&, const string&);

	string pathNormalize(const string&);

	template<class T> string toString(const T& t);

	template<> string toString(const bool&);

	template<> string toString(const double&);

	template<> string toString(const float&);

	template<class T> T fromString(const string&);

	template<> double fromString(const string&);

	template<> float fromString(const string&);

	string pad(const string&, int size);

	string toString(double, int precision);

	string toLower(const string&);

	string toUpper(const string&);

	string ltrim(const string&, const string& chars = " \t");

	string rtrim(const string&, const string& chars = " \t");

	string trim(const string&, const string& chars = " \t");

	string replaceAll(const string& str, const string& from, const string& to, bool aggressive = false);

	string tmpPath();

	string createTmpFilePath();

	string fileExt(const string& path);

	void removeFile(const string& path);

	int readIntValue(const string& data, const string& match);

	string readStringValue(const string& data, const string& match);

	vector<string>& split(const string&, char delim, vector<string> &elems);
	vector<string> split(const string&, char delim);

	namespace Time
	{
		clock_t now();

		double elapsed(clock_t now, clock_t startTime);

		double elapsed(clock_t now);

		double elapsed();

		string toString(double elapsed);

		string toString();

	} /* namespace Time */


	/* implemtation */

	template<class T> string toString(const T& t)
	{
		string str;
		stringstream ss;
		ss << t;
		ss >> str;
		return str;
	}

	template<class T> T fromString(const string& str)
	{
		T t;
		stringstream ss;
		ss << str;
		ss >> t;
		return t;
	}

}

#endif /* UTILS_H */
