/**
 * SpritePack
 * Copyright (C) 2015 SpritePack creators
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef VHTL_H
#define VHTL_H

#include "bucketmove.h"
#include "point.h"
#include "geometry.h"

namespace SpritePack {

	class VHTL : public GeometryBucketMove
	{
	public:
		virtual Cost calc();

		virtual string name() const;

	protected:

		struct SPoint
		{
			Point arrayPosition;

			Point realPosition;

			SPoint(const Point& arrayPosition = Point(0, 0), const Point& realPosition = Point(0, 0));
		};

		struct S
		{
			Bucket::Items items;
			vector<int> columns;
			vector<int> rows;
			vector< vector<bool> > mask;

			Base * base;

			S(Base* = 0, const Geometry& = Geometry());
			
			bool isValid(const SPoint&) const;

			Geometry unfittedGeometry(const SPoint&, Id id) const; //if < 0, result is -freespace after insertion
			
			void insert(const SPoint&, Id id);

			void visualize() const;

			struct Cmp
			{
				bool operator()(const pair<Area, SPoint>& a, const pair<Area, SPoint>& b) const
				{
					if (a.first != b.first)
					{
						return a.first > b.first;
					}
					else if (a.second.arrayPosition.x != b.second.arrayPosition.x)
					{
						return a.second.arrayPosition.x < b.second.arrayPosition.x;
					}
					else
					{
						return a.second.arrayPosition.y < b.second.arrayPosition.y;
					}
				}
			};

			set< pair<Area, SPoint>, Cmp> m;

			void recreateM();

			Area calcFreeArea(const SPoint&) const;
		};

		struct ArrangeItemsResult
		{
			Bucket::Items items;
			Geometry minUnfittedGeometry;
			Id unfittedId;

			bool hasUnfitted() const;
			ArrangeItemsResult(const Bucket::Items& = Bucket::Items(), const Geometry& = Geometry(0, 0), Id unfittedId = -1);
		};

		struct DecreasingHeightCmp
		{
			Base * base;
			DecreasingHeightCmp(Base *);

			bool operator()(Id, Id) const;
		};

	protected:
		virtual ArrangeItemsResult arrangeItems(const vector<Id>& ids, const Geometry&) const; 

		virtual int getWidthSum(const vector<Id>&) const;

		virtual int getHeightSum(const vector<Id>&) const;

		virtual int getMaxHeight(const vector<Id>&) const;

		virtual int getMaxWidth(const vector<Id>&) const;

		virtual Area getAreasSum(const vector<Id>&) const;

		virtual bool isBetterPoint(const SPoint& candidate, const SPoint& currentBest, const Geometry& areaGeometry, const Geometry& itemGeometry) const;
	};


} // namespace SpritePack

#endif // VHTL_H